@@ -20,6 +20,8 @@
 
 #include "z-file.h"
 
+#define PLAYER_NAME_LEN		32
+
 /**
  * Option types 
  */
@@ -39,35 +41,46 @@ enum
  */
 enum
 {
-    #define OP(a, b, c, d) OPT_##a,
-    #include "list-options.h"
-    #undef OP
+	#define OP(a, b, c, d) OPT_##a,
+	#include "list-options.h"
+	#undef OP
 	OPT_MAX
 };
 
-
-#define OPT(opt_name)	op_ptr->opt[OPT_##opt_name]
+#define OPT(p, opt_name)	p->opts.opt[OPT_##opt_name]
 
 /**
  * Information for "do_cmd_options()".
  */
 #define OPT_PAGE_MAX				OP_SCORE
-#define OPT_PAGE_PER				20
+#define OPT_PAGE_PER				21
 #define OPT_PAGE_BIRTH				1
 
 /**
  * The option data structures
  */
+struct player_options {
+	bool opt[OPT_MAX];		/**< Options */
+
+	byte hitpoint_warn;		/**< Hitpoint warning (0 to 9) */
+	u16b lazymove_delay;	/**< Delay in cs before moving to allow another key */
+	byte delay_factor;		/**< Delay factor (0 to 9) */
+
+	byte name_suffix;		/**< Numeric suffix for player name */
+};
+
 extern int option_page[OPT_PAGE_MAX][OPT_PAGE_PER];
 
 /**
- * Functions 
-*/
+ * Functions
+ */
+void options_init_cheat(void);
 const char *option_name(int opt);
 const char *option_desc(int opt);
 int option_type(int opt);
 bool option_set(const char *opt, int val);
+void options_init_cheat(void);
+void options_init_defaults(struct player_options *opts);
 void init_options(void);
 
-
 #endif /* !INCLUDED_OPTIONS_H */
