@@ -24,12 +24,15 @@
 #include "init.h"
 #include "mon-lore.h"
 #include "monster.h"
+#include "obj-curse.h"
 #include "obj-gear.h"
-#include "obj-identify.h"
 #include "obj-ignore.h"
+#include "obj-init.h"
+#include "obj-knowledge.h"
 #include "obj-make.h"
 #include "obj-pile.h"
 #include "obj-power.h"
+#include "obj-properties.h"
 #include "obj-randart.h"
 #include "obj-tval.h"
 #include "obj-util.h"
@@ -102,6 +105,7 @@ struct birther
 	s16b stat[STAT_MAX];
 
 	char *history;
+	char name[PLAYER_NAME_LEN];
 };
 
 
@@ -116,7 +120,7 @@ static int points_spent[STAT_MAX];
 static int points_left;
 
 static bool quickstart_allowed;
-static bool rolled_stats = FALSE;
+static bool rolled_stats = false;
 
 /**
  * The last character displayed, to allow the user to flick between two.
@@ -155,6 +159,7 @@ static void save_roller_data(birther *tosave)
 		tosave->stat[i] = player->stat_birth[i];
 
 	tosave->history = player->history;
+	my_strcpy(tosave->name, player->full_name, sizeof(tosave->name));
 }
 
 
@@ -193,10 +198,12 @@ static void load_roller_data(birther *saved, birther *prev_player)
 	for (i = 0; i < STAT_MAX; i++) {
 		player->stat_max[i] = player->stat_cur[i] = player->stat_birth[i]
 			= saved->stat[i];
+		player->stat_map[i] = i;
 	}
 
 	/* Load previous history */
 	player->history = saved->history;
+	my_strcpy(player->full_name, saved->name, sizeof(player->full_name));
 
 	/* Save the current data if the caller is interested in it. */
 	if (prev_player)
@@ -216,7 +223,7 @@ static void get_stats(int stat_use[STAT_MAX])
 	int dice[3 * STAT_MAX];
 
 	/* Roll and verify some stats */
-	while (TRUE) {
+	while (true) {
 		/* Roll some dice */
 		for (j = i = 0; i < 3 * STAT_MAX; i++) {
 			/* Roll the dice */
@@ -246,6 +253,9 @@ static void get_stats(int stat_use[STAT_MAX])
 		/* Start fully healed */
 		player->stat_cur[i] = player->stat_max[i];
 
+		/* Start with unscrambled stats */
+		player->stat_map[i] = i;
+
 		/* Efficiency -- Apply the racial/class bonuses */
 		stat_use[i] = modify_stat_value(player->stat_max[i], bonus);
 
@@ -267,7 +277,7 @@ static void roll_hp(void)
 	max_value += PY_MAX_LEVEL;
 
 	/* Roll out the hitpoints */
-	while (TRUE) {
+	while (true) {
 		/* Roll the hitpoint values */
 		for (i = 1; i < PY_MAX_LEVEL; i++) {
 			j = randint1(player->hitdie);
@@ -341,6 +351,27 @@ static void get_ahw(struct player *p)
 
 
 
+/**
+ * Creates the player's body
+ */
+static void player_embody(struct player *p)
+{
+	char buf[80];
+	int i;
+
+	assert(p->race);
+
+	memcpy(&p->body, &bodies[p->race->body], sizeof(p->body));
+	my_strcpy(buf, bodies[p->race->body].name, sizeof(buf));
+	p->body.name = string_make(buf);
+	p->body.slots = mem_zalloc(p->body.count * sizeof(struct equip_slot));
+	for (i = 0; i < p->body.count; i++) {
+		p->body.slots[i].type = bodies[p->race->body].slots[i].type;
+		my_strcpy(buf, bodies[p->race->body].slots[i].name, sizeof(buf));
+		p->body.slots[i].name = string_make(buf);
+	}
+}
+
 /**
  * Get the player's starting money
  */
@@ -352,6 +383,7 @@ static void get_money(void)
 void player_init(struct player *p)
 {
 	int i;
+	struct player_options opts_save = p->opts;
 
 	if (p->upkeep) {
 		if (p->upkeep->inven)
@@ -362,6 +394,12 @@ void player_init(struct player *p)
 	}
 	if (p->timed)
 		mem_free(p->timed);
+	if (p->obj_k) {
+		mem_free(p->obj_k->brands);
+		mem_free(p->obj_k->slays);
+		mem_free(p->obj_k->curses);
+		mem_free(p->obj_k);
+	}
 
 	/* Wipe the player */
 	memset(p, 0, sizeof(struct player));
@@ -369,8 +407,8 @@ void player_init(struct player *p)
 	/* Start with no artifacts made yet */
 	for (i = 0; z_info && i < z_info->a_max; i++) {
 		struct artifact *art = &a_info[i];
-		art->created = FALSE;
-		art->seen = FALSE;
+		art->created = false;
+		art->seen = false;
 	}
 
 	/* Start with no quests */
@@ -378,8 +416,8 @@ void player_init(struct player *p)
 
 	for (i = 1; z_info && i < z_info->k_max; i++) {
 		struct object_kind *kind = &k_info[i];
-		kind->tried = FALSE;
-		kind->aware = FALSE;
+		kind->tried = false;
+		kind->aware = false;
 	}
 
 	for (i = 1; z_info && i < z_info->r_max; i++) {
@@ -401,6 +439,14 @@ void player_init(struct player *p)
 	p->upkeep->quiver = mem_zalloc(z_info->quiver_size *
 								   sizeof(struct object *));
 	p->timed = mem_zalloc(TMD_MAX * sizeof(s16b));
+	p->obj_k = mem_zalloc(sizeof(struct object));
+	p->obj_k->brands = mem_zalloc(z_info->brand_max * sizeof(bool));
+	p->obj_k->slays = mem_zalloc(z_info->slay_max * sizeof(bool));
+	p->obj_k->curses = mem_zalloc(z_info->curse_max *
+								  sizeof(struct curse_data));
+
+	/* Options should persist */
+	p->opts = opts_save;
 
 	/* First turn. */
 	turn = 1;
@@ -417,7 +463,7 @@ void player_init(struct player *p)
  */
 void wield_all(struct player *p)
 {
-	struct object *obj, *new_pile = NULL;
+	struct object *obj, *new_pile = NULL, *new_known_pile = NULL;
 	int slot;
 
 	/* Scan through the slots */
@@ -443,19 +489,22 @@ void wield_all(struct player *p)
 
 			/* Add to the pile of new objects to carry */
 			pile_insert(&new_pile, new);
+			pile_insert(&new_known_pile, new->known);
 		}
 
 		/* Wear the new stuff */
 		p->body.slots[slot].obj = obj;
+		object_learn_on_wield(p, obj);
 
 		/* Increment the equip counter by hand */
 		p->upkeep->equip_cnt++;
 	}
 
 	/* Now add the unwielded split objects to the gear */
-	if (new_pile)
+	if (new_pile) {
 		pile_insert_end(&player->gear, new_pile);
-
+		pile_insert_end(&player->gear_k, new_known_pile);
+	}
 	return;
 }
 
@@ -467,53 +516,55 @@ void wield_all(struct player *p)
  */
 static void player_outfit(struct player *p)
 {
-	char buf[80];
 	int i;
 	const struct start_item *si;
-
-	/* Player needs a body */
-	memcpy(&p->body, &bodies[p->race->body], sizeof(p->body));
-	my_strcpy(buf, bodies[p->race->body].name, sizeof(buf));
-	p->body.name = string_make(buf);
-	p->body.slots = mem_zalloc(p->body.count * sizeof(struct equip_slot));
-	for (i = 0; i < p->body.count; i++) {
-		p->body.slots[i].type = bodies[p->race->body].slots[i].type;
-		my_strcpy(buf, bodies[p->race->body].slots[i].name, sizeof(buf));
-		p->body.slots[i].name = string_make(buf);
-	}
+	struct object *obj, *known_obj;
 
 	/* Currently carrying nothing */
 	p->upkeep->total_weight = 0;
 
+	/* Give the player obvious object knowledge */
+	p->obj_k->dd = 1;
+	p->obj_k->ds = 1;
+	p->obj_k->ac = 1;
+	for (i = 1; i < OF_MAX; i++) {
+		struct obj_property *prop = lookup_obj_property(OBJ_PROPERTY_FLAG, i);
+		if (prop->subtype == OFT_LIGHT) of_on(p->obj_k->flags, i);
+		if (prop->subtype == OFT_DIG) of_on(p->obj_k->flags, i);
+	}
+
 	/* Give the player starting equipment */
 	for (si = p->class->start_items; si; si = si->next) {
-		/* Get local object */
-		struct object *obj = object_new();
 		int num = rand_range(si->min, si->max);
 
 		/* Without start_kit, only start with 1 food and 1 light */
-		if (!OPT(birth_start_kit)) {
+		if (!OPT(p, birth_start_kit)) {
 			if (!tval_is_food_k(si->kind) && !tval_is_light_k(si->kind))
 				continue;
 
 			num = 1;
 		}
 
-		/* Prepare the item */
+		/* Prepare a new item */
+		obj = object_new();
 		object_prep(obj, si->kind, 0, MINIMISE);
 		obj->number = num;
 		obj->origin = ORIGIN_BIRTH;
 
+		known_obj = object_new();
+		obj->known = known_obj;
+		object_set_base_known(obj);
 		object_flavor_aware(obj);
-		object_notice_everything(obj);
-		apply_autoinscription(obj);
+		obj->known->pval = obj->pval;
+		obj->known->effect = obj->effect;
+		obj->known->notice |= OBJ_NOTICE_ASSESSED;
 
 		/* Deduct the cost of the item from starting cash */
-		p->au -= object_value(obj, obj->number, FALSE);
+		p->au -= object_value_real(obj, obj->number);
 
 		/* Carry the item */
-		inven_carry(p, obj, TRUE, FALSE);
-		si->kind->everseen = TRUE;
+		inven_carry(p, obj, true, false);
+		si->kind->everseen = true;
 	}
 
 	/* Sanity check */
@@ -522,6 +573,9 @@ static void player_outfit(struct player *p)
 
 	/* Now try wielding everything */
 	wield_all(p);
+
+	/* Update knowledge */
+	update_player_object_knowledge(p);
 }
 
 
@@ -533,17 +587,19 @@ static const int birth_stat_costs[18 + 1] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
 /* It was feasible to get base 17 in 3 stats with the autoroller */
 #define MAX_BIRTH_POINTS 20 /* 3 * (1+1+1+1+1+1+2) */
 
-static void recalculate_stats(int *stats, int points_left)
+static void recalculate_stats(int *stats_local_local, int points_left_local)
 {
 	int i;
 
 	/* Variable stat maxes */
-	for (i = 0; i < STAT_MAX; i++)
+	for (i = 0; i < STAT_MAX; i++) {
 		player->stat_cur[i] = player->stat_max[i] =
-				player->stat_birth[i] = stats[i];
+				player->stat_birth[i] = stats_local_local[i];
+		player->stat_map[i] = i;
+	}
 
 	/* Gold is inversely proportional to cost */
-	player->au_birth = z_info->start_gold + (50 * points_left);
+	player->au_birth = z_info->start_gold + (50 * points_left_local);
 
 	/* Update bonuses, hp, etc. */
 	get_bonuses();
@@ -555,87 +611,87 @@ static void recalculate_stats(int *stats, int points_left)
 	event_signal(EVENT_STATS);
 }
 
-static void reset_stats(int stats[STAT_MAX], int points_spent[STAT_MAX],
-						int *points_left, bool update_display)
+static void reset_stats(int stats_local[STAT_MAX], int points_spent_local_local[STAT_MAX],
+						int *points_left_local, bool update_display)
 {
 	int i;
 
 	/* Calculate and signal initial stats and points totals. */
-	*points_left = MAX_BIRTH_POINTS;
+	*points_left_local = MAX_BIRTH_POINTS;
 
 	for (i = 0; i < STAT_MAX; i++) {
 		/* Initial stats are all 10 and costs are zero */
-		stats[i] = 10;
-		points_spent[i] = 0;
+		stats_local[i] = 10;
+		points_spent_local_local[i] = 0;
 	}
 
 	/* Use the new "birth stat" values to work out the "other"
 	   stat values (i.e. after modifiers) and tell the UI things have 
 	   changed. */
 	if (update_display) {
-		recalculate_stats(stats, *points_left);
-		event_signal_birthpoints(points_spent, *points_left);	
+		recalculate_stats(stats_local, *points_left_local);
+		event_signal_birthpoints(points_spent_local_local, *points_left_local);	
 	}
 }
 
-static bool buy_stat(int choice, int stats[STAT_MAX],
-					 int points_spent[STAT_MAX], int *points_left,
+static bool buy_stat(int choice, int stats_local[STAT_MAX],
+					 int points_spent_local[STAT_MAX], int *points_left_local,
 					 bool update_display)
 {
 	/* Must be a valid stat, and have a "base" of below 18 to be adjusted */
-	if (!(choice >= STAT_MAX || choice < 0) &&	(stats[choice] < 18)) {
+	if (!(choice >= STAT_MAX || choice < 0) &&	(stats_local[choice] < 18)) {
 		/* Get the cost of buying the extra point (beyond what
 		   it has already cost to get this far). */
-		int stat_cost = birth_stat_costs[stats[choice] + 1];
+		int stat_cost = birth_stat_costs[stats_local[choice] + 1];
 
-		if (stat_cost <= *points_left) {
-			stats[choice]++;
-			points_spent[choice] += stat_cost;
-			*points_left -= stat_cost;
+		if (stat_cost <= *points_left_local) {
+			stats_local[choice]++;
+			points_spent_local[choice] += stat_cost;
+			*points_left_local -= stat_cost;
 
 			if (update_display) {
 				/* Tell the UI the new points situation. */
-				event_signal_birthpoints(points_spent, *points_left);
+				event_signal_birthpoints(points_spent_local, *points_left_local);
 
 				/* Recalculate everything that's changed because
 				   the stat has changed, and inform the UI. */
-				recalculate_stats(stats, *points_left);
+				recalculate_stats(stats_local, *points_left_local);
 			}
 
-			return TRUE;
+			return true;
 		}
 	}
 
 	/* Didn't adjust stat. */
-	return FALSE;
+	return false;
 }
 
 
-static bool sell_stat(int choice, int stats[STAT_MAX], int points_spent[STAT_MAX],
-	int *points_left, bool update_display)
+static bool sell_stat(int choice, int stats_local[STAT_MAX], int points_spent_local[STAT_MAX],
+	int *points_left_local, bool update_display)
 {
 	/* Must be a valid stat, and we can't "sell" stats below the base of 10. */
-	if (!(choice >= STAT_MAX || choice < 0) && (stats[choice] > 10)) {
-		int stat_cost = birth_stat_costs[stats[choice]];
+	if (!(choice >= STAT_MAX || choice < 0) && (stats_local[choice] > 10)) {
+		int stat_cost = birth_stat_costs[stats_local[choice]];
 
-		stats[choice]--;
-		points_spent[choice] -= stat_cost;
-		*points_left += stat_cost;
+		stats_local[choice]--;
+		points_spent_local[choice] -= stat_cost;
+		*points_left_local += stat_cost;
 
 		if (update_display) {
 			/* Tell the UI the new points situation. */
-			event_signal_birthpoints(points_spent, *points_left);
+			event_signal_birthpoints(points_spent_local, *points_left_local);
 
 			/* Recalculate everything that's changed because
 			   the stat has changed, and inform the UI. */
-			recalculate_stats(stats, *points_left);
+			recalculate_stats(stats_local, *points_left_local);
 	
-			return TRUE;
+			return true;
 		}
 	}
 
 	/* Didn't adjust stat. */
-	return FALSE;
+	return false;
 }
 
 
@@ -653,35 +709,36 @@ static bool sell_stat(int choice, int stats[STAT_MAX], int points_spent[STAT_MAX
  * 3. If there are any points left, spend as much as possible in order 
  *    on DEX and then the non-spell-stat.
  */
-static void generate_stats(int stats[STAT_MAX], int points_spent[STAT_MAX], 
-						   int *points_left)
+static void generate_stats(int stats_local[STAT_MAX], int points_spent_local[STAT_MAX], 
+						   int *points_left_local)
 {
 	int step = 0;
 	int maxed[STAT_MAX] = { 0 };
-	int spell_stat = player->class->magic.spell_realm->stat;
-	bool caster = FALSE, warrior = FALSE;
+	int spell_stat = player->class->magic.spell_realm ? 
+		player->class->magic.spell_realm->stat : 0;
+	bool caster = false, warrior = false;
 
 	/* Determine whether the class is warrior */
 	if (player->class->max_attacks > 5) { 
-		warrior = TRUE;
+		warrior = true;
 	}
 
 	/* Determine whether the class is priest/mage */
 	if (player->class->max_attacks < 5) {
-		caster = TRUE;
+		caster = true;
 	}
 
-	while (*points_left && step >= 0) {
+	while (*points_left_local && step >= 0) {
 	
 		switch (step) {
 		
 			/* Buy base STR 17 */
 			case 0: {
 			
-				if (!maxed[STAT_STR] && stats[STAT_STR] < 17) {
-					if (!buy_stat(STAT_STR, stats, points_spent, points_left,
-								  FALSE))
-						maxed[STAT_STR] = TRUE;
+				if (!maxed[STAT_STR] && stats_local[STAT_STR] < 17) {
+					if (!buy_stat(STAT_STR, stats_local, points_spent_local, points_left_local,
+								  false))
+						maxed[STAT_STR] = true;
 				} else {
 					step++;
 					
@@ -698,9 +755,9 @@ static void generate_stats(int stats[STAT_MAX], int points_spent[STAT_MAX],
 			case 1: {
 				if (!maxed[STAT_DEX] && player->state.stat_top[STAT_DEX]
 					< 18+10) {
-					if (!buy_stat(STAT_DEX, stats, points_spent, points_left,
-								  FALSE))
-						maxed[STAT_DEX] = TRUE;
+					if (!buy_stat(STAT_DEX, stats_local, points_spent_local, points_left_local,
+								  false))
+						maxed[STAT_DEX] = true;
 				} else {
 					step++;
 				}
@@ -711,10 +768,10 @@ static void generate_stats(int stats[STAT_MAX], int points_spent[STAT_MAX],
 			/* If we can't get 18/10 dex, sell it back. */
 			case 2: {
 				if (player->state.stat_top[STAT_DEX] < 18+10) {
-					while (stats[STAT_DEX] > 10)
-						sell_stat(STAT_DEX, stats, points_spent, points_left,
-								  FALSE);
-					maxed[STAT_DEX] = FALSE;
+					while (stats_local[STAT_DEX] > 10)
+						sell_stat(STAT_DEX, stats_local, points_spent_local, points_left_local,
+								  false);
+					maxed[STAT_DEX] = false;
 				}
 				step++;
 			}
@@ -726,25 +783,25 @@ static void generate_stats(int stats[STAT_MAX], int points_spent[STAT_MAX],
 			 */
 			case 3: 
 			{
-				int points_trigger = *points_left / 2;
+				int points_trigger = *points_left_local / 2;
 				
 				if (warrior) {
-					points_trigger = *points_left;
+					points_trigger = *points_left_local;
 				} else {
 					while (!maxed[spell_stat] &&
-						   (caster || stats[spell_stat] < 16) &&
-						   points_spent[spell_stat] < points_trigger) {
+						   (caster || stats_local[spell_stat] < 16) &&
+						   points_spent_local[spell_stat] < points_trigger) {
 
-						if (!buy_stat(spell_stat, stats, points_spent,
-									  points_left, FALSE)) {
-							maxed[spell_stat] = TRUE;
+						if (!buy_stat(spell_stat, stats_local, points_spent_local,
+									  points_left_local, false)) {
+							maxed[spell_stat] = true;
 						}
 
-						if (points_spent[spell_stat] > points_trigger) {
+						if (points_spent_local[spell_stat] > points_trigger) {
 						
-							sell_stat(spell_stat, stats, points_spent, 
-									  points_left, FALSE);
-							maxed[spell_stat] = TRUE;
+							sell_stat(spell_stat, stats_local, points_spent_local, 
+									  points_left_local, false);
+							maxed[spell_stat] = true;
 						}
 					}
 				}
@@ -752,18 +809,18 @@ static void generate_stats(int stats[STAT_MAX], int points_spent[STAT_MAX],
 				/* Skip CON for casters because DEX is more important early
 				 * and is handled in 4 */
 				while (!maxed[STAT_CON] &&
-					   !(caster) && stats[STAT_CON] < 16 &&
-					   points_spent[STAT_CON] < points_trigger) {
+					   !(caster) && stats_local[STAT_CON] < 16 &&
+					   points_spent_local[STAT_CON] < points_trigger) {
 					   
-					if (!buy_stat(STAT_CON, stats, points_spent,points_left,
-								  FALSE)) {
-						maxed[STAT_CON] = TRUE;
+					if (!buy_stat(STAT_CON, stats_local, points_spent_local,points_left_local,
+								  false)) {
+						maxed[STAT_CON] = true;
 					}
 
-					if (points_spent[STAT_CON] > points_trigger) {
-						sell_stat(STAT_CON, stats, points_spent, points_left,
-								  FALSE);
-						maxed[STAT_CON] = TRUE;
+					if (points_spent_local[STAT_CON] > points_trigger) {
+						sell_stat(STAT_CON, stats_local, points_spent_local, points_left_local,
+								  false);
+						maxed[STAT_CON] = true;
 					}
 				}
 				
@@ -791,9 +848,9 @@ static void generate_stats(int stats[STAT_MAX], int points_spent[STAT_MAX],
 				}
 
 				/* Buy until we can't buy any more. */
-				while (buy_stat(next_stat, stats, points_spent, points_left,
-								FALSE));
-				maxed[next_stat] = TRUE;
+				while (buy_stat(next_stat, stats_local, points_spent_local, points_left_local,
+								false));
+				maxed[next_stat] = true;
 
 				break;
 			}
@@ -806,11 +863,11 @@ static void generate_stats(int stats[STAT_MAX], int points_spent[STAT_MAX],
 		}
 	}
 	/* Tell the UI the new points situation. */
-	event_signal_birthpoints(points_spent, *points_left);
+	event_signal_birthpoints(points_spent_local, *points_left_local);
 
 	/* Recalculate everything that's changed because
 	   the stat has changed, and inform the UI. */
-	recalculate_stats(stats, *points_left);
+	recalculate_stats(stats_local, *points_left_local);
 }
 
 /**
@@ -854,14 +911,14 @@ void player_generate(struct player *p, const struct player_race *r,
 /**
  * Reset everything back to how it would be on loading the game.
  */
-static void do_birth_reset(bool use_quickstart, birther *quickstart_prev)
+static void do_birth_reset(bool use_quickstart, birther *quickstart_prev_local)
 {
 	/* If there's quickstart data, we use it to set default
 	   character choices. */
-	if (use_quickstart && quickstart_prev)
-		load_roller_data(quickstart_prev, NULL);
+	if (use_quickstart && quickstart_prev_local)
+		load_roller_data(quickstart_prev_local, NULL);
 
-	player_generate(player, NULL, NULL, use_quickstart && quickstart_prev);
+	player_generate(player, NULL, NULL, use_quickstart && quickstart_prev_local);
 
 	player->depth = 0;
 
@@ -874,7 +931,7 @@ void do_cmd_birth_init(struct command *cmd)
 	char *buf;
 
 	/* The dungeon is not ready */
-	character_dungeon = FALSE;
+	character_dungeon = false;
 
 	/*
 	 * If there's a quickstart character, store it for later use.
@@ -882,23 +939,28 @@ void do_cmd_birth_init(struct command *cmd)
 	 */
 	if (player->ht_birth) {
 		save_roller_data(&quickstart_prev);
-		quickstart_allowed = TRUE;
+		quickstart_allowed = true;
 	} else {
-		player_generate(player, player_id2race(0), player_id2class(0), FALSE);
-		quickstart_allowed = FALSE;
+		player_generate(player, player_id2race(0), player_id2class(0), false);
+		quickstart_allowed = false;
 	}
 
 	/* Handle incrementing name suffix */
-	buf = find_roman_suffix_start(op_ptr->full_name);
+	buf = find_roman_suffix_start(player->full_name);
 	if (buf) {
 		/* Try to increment the roman suffix */
-		int success = int_to_roman((roman_to_int(buf) + 1), buf,
-			(sizeof(op_ptr->full_name) - (buf -
-			(char *)&op_ptr->full_name)));
-			
-		if (!success) msg("Sorry, could not deal with suffix");
+		int success = int_to_roman(
+				roman_to_int(buf) + 1,
+				buf,
+				sizeof(player->full_name) - (buf - (char *)&player->full_name));
+
+		if (success) {
+			save_roller_data(&quickstart_prev);
+		} else {
+			msg("Sorry, could not deal with suffix");
+		}
 	}
-	
+
 	/* We're ready to start the birth process */
 	event_signal_flag(EVENT_ENTER_BIRTH, quickstart_allowed);
 }
@@ -906,31 +968,31 @@ void do_cmd_birth_init(struct command *cmd)
 void do_cmd_birth_reset(struct command *cmd)
 {
 	player_init(player);
-	reset_stats(stats, points_spent, &points_left, FALSE);
+	reset_stats(stats, points_spent, &points_left, false);
 	do_birth_reset(quickstart_allowed, &quickstart_prev);
-	rolled_stats = FALSE;
+	rolled_stats = false;
 }
 
 void do_cmd_choose_race(struct command *cmd)
 {
 	int choice;
 	cmd_get_arg_choice(cmd, "choice", &choice);
-	player_generate(player, player_id2race(choice), NULL, FALSE);
+	player_generate(player, player_id2race(choice), NULL, false);
 
-	reset_stats(stats, points_spent, &points_left, FALSE);
+	reset_stats(stats, points_spent, &points_left, false);
 	generate_stats(stats, points_spent, &points_left);
-	rolled_stats = FALSE;
+	rolled_stats = false;
 }
 
 void do_cmd_choose_class(struct command *cmd)
 {
 	int choice;
 	cmd_get_arg_choice(cmd, "choice", &choice);
-	player_generate(player, NULL, player_id2class(choice), FALSE);
+	player_generate(player, NULL, player_id2class(choice), false);
 
-	reset_stats(stats, points_spent, &points_left, FALSE);
+	reset_stats(stats, points_spent, &points_left, false);
 	generate_stats(stats, points_spent, &points_left);
-	rolled_stats = FALSE;
+	rolled_stats = false;
 }
 
 void do_cmd_buy_stat(struct command *cmd)
@@ -939,7 +1001,7 @@ void do_cmd_buy_stat(struct command *cmd)
 	if (!rolled_stats) {
 		int choice;
 		cmd_get_arg_choice(cmd, "choice", &choice);
-		buy_stat(choice, stats, points_spent, &points_left, TRUE);
+		buy_stat(choice, stats, points_spent, &points_left, true);
 	}
 }
 
@@ -949,7 +1011,7 @@ void do_cmd_sell_stat(struct command *cmd)
 	if (!rolled_stats) {
 		int choice;
 		cmd_get_arg_choice(cmd, "choice", &choice);
-		sell_stat(choice, stats, points_spent, &points_left, TRUE);
+		sell_stat(choice, stats, points_spent, &points_left, true);
 	}
 }
 
@@ -958,13 +1020,13 @@ void do_cmd_reset_stats(struct command *cmd)
 	/* .choice is whether to regen stats */
 	int choice;
 
-	reset_stats(stats, points_spent, &points_left, TRUE);
+	reset_stats(stats, points_spent, &points_left, true);
 
 	cmd_get_arg_choice(cmd, "choice", &choice);
 	if (choice)
 		generate_stats(stats, points_spent, &points_left);
 
-	rolled_stats = FALSE;
+	rolled_stats = false;
 }
 
 void do_cmd_roll_stats(struct command *cmd)
@@ -981,6 +1043,8 @@ void do_cmd_roll_stats(struct command *cmd)
 
 	/* There's no real need to do this here, but it's tradition. */
 	get_ahw(player);
+	if (player->history)
+		string_free(player->history);
 	player->history = get_history(player->race->history);
 
 	event_signal(EVENT_GOLD);
@@ -996,7 +1060,7 @@ void do_cmd_roll_stats(struct command *cmd)
 	event_signal_birthpoints(points_spent, points_left);
 
 	/* Lock out buying and selling of stats based on rolled stats. */
-	rolled_stats = TRUE;
+	rolled_stats = true;
 }
 
 void do_cmd_prev_stats(struct command *cmd)
@@ -1020,7 +1084,7 @@ void do_cmd_choose_name(struct command *cmd)
 	cmd_get_arg_string(cmd, "name", &str);
 
 	/* Set player name */
-	my_strcpy(op_ptr->full_name, str, sizeof(op_ptr->full_name));
+	my_strcpy(player->full_name, str, sizeof(player->full_name));
 
 	string_free((char *) str);
 }
@@ -1042,21 +1106,15 @@ void do_cmd_choose_history(struct command *cmd)
 
 void do_cmd_accept_character(struct command *cmd)
 {
-	int i;
-
-	/* Reset score options from cheat options */
-	for (i = 0; i < OPT_MAX; i++) {
-		if (option_type(i) == OP_CHEAT)
-			op_ptr->opt[i + 1] = op_ptr->opt[i];
-	}
+	options_init_cheat();
 
 	roll_hp();
 
 	ignore_birth_init();
 
 	/* Clear old messages, add new starting message */
-	history_clear();
-	history_add("Began the quest to destroy Morgoth.", HIST_PLAYER_BIRTH, 0);
+	history_clear(player);
+	history_add(player, "Began the quest to destroy Morgoth.", HIST_PLAYER_BIRTH);
 
 	/* Note player birth in the message recall */
 	message_add(" ", MSG_GENERIC);
@@ -1065,36 +1123,59 @@ void do_cmd_accept_character(struct command *cmd)
 	message_add("  ", MSG_GENERIC);
 	message_add(" ", MSG_GENERIC);
 
+	/* Embody */
+	player_embody(player);
+
 	/* Give the player some money */
 	get_money();
 
-	/* Outfit the player, if they can sell the stuff */
-	player_outfit(player);
-
 	/* Initialise the spells */
 	player_spells_init(player);
 
+	/* Know all runes for ID on walkover */
+	if (OPT(player, birth_know_runes))
+		player_learn_all_runes(player);
+
+	/* Hack - player knows all combat runes.  Maybe make them not runes? NRM */
+	player->obj_k->to_a = 1;
+	player->obj_k->to_h = 1;
+	player->obj_k->to_d = 1;
+
 	/* Initialise the stores */
 	store_reset();
 
-	/* Seed for random artifacts */
-	if (!seed_randart || !OPT(birth_keep_randarts))
-		seed_randart = randint0(0x10000000);
+	/* Player learns innate runes */
+	player_learn_innate(player);
 
 	/* Randomize the artifacts if required */
-	if (OPT(birth_randarts))
-		do_randart(seed_randart, TRUE);
+	if (OPT(player, birth_randarts)) {
+		/* First restore the standard artifacts */
+		cleanup_parser(&randart_parser);
+		deactivate_randart_file();
+		run_parser(&artifact_parser);
+
+		/* Now generate the new randarts */
+		seed_randart = randint0(0x10000000);
+		do_randart(seed_randart, true);
+	}
 
 	/* Seed for flavors */
 	seed_flavor = randint0(0x10000000);
 	flavor_init();
 
+	/* Know all flavors for auto-ID of consumables */
+	if (OPT(player, birth_know_flavors))
+		flavor_set_all_aware();
+
+	/* Outfit the player, if they can sell the stuff */
+	player_outfit(player);
+
 	/* Stop the player being quite so dead */
-	player->is_dead = FALSE;
+	player->is_dead = false;
 
 	/* Character is now "complete" */
-	character_generated = TRUE;
-	player->upkeep->playing = TRUE;
+	character_generated = true;
+	player->upkeep->playing = true;
 
 	/* Disable repeat command, so we don't try to be born again */
 	cmd_disable_repeat();
