@@ -18,15 +18,24 @@ int setup_tests(void **state) {
 }
 
 int teardown_tests(void *state) {
+	struct player *p = state;
 	player_quests_free((struct player *)state);
 	mem_free(z_info);
+	mem_free(p->upkeep->inven);
+	mem_free(p->upkeep->quiver);
+	mem_free(p->upkeep);
+	mem_free(p->timed);
+	mem_free(p->obj_k->brands);
+	mem_free(p->obj_k->slays);
+	mem_free(p->obj_k->curses);
+	mem_free(p->obj_k);
 	mem_free(state);
 	return 0;
 }
 
 int test_generate0(void *state) {
 	struct player *p = state;
-	player_generate(p, &test_race, &test_class, FALSE);
+	player_generate(p, &test_race, &test_class, false);
 	eq(p->lev, 1);
 	ptreq(p->race, &test_race);
 	ptreq(p->class, &test_class);
